@@ -19,6 +19,7 @@
 #include "ivutils.h"
 #include <OpenImageIO/strutil.h>
 #include <OpenImageIO/timer.h>
+#include <cfloat>
 
 OIIO_PRAGMA_WARNING_PUSH
 #if defined(__APPLE__)
@@ -189,19 +190,18 @@ IvGL::create_textures(void)
 const char*
 IvGL::color_func_shader_text()
 {
-    // clang-format off
-    return
-        "uniform float gain;\n"
-        "uniform float gamma;\n"
-        "\n"
-        "vec4 ColorFunc(vec4 C)\n"
-        "{\n"
-        "    C.xyz *= gain;\n"
-        "    float invgamma = 1.0/gamma;\n"
-        "    C.xyz = pow (C.xyz, vec3 (invgamma, invgamma, invgamma));\n"
-        "    return C;\n"
-        "}\n";
-    // clang-format on
+    return R"glsl(
+        uniform float gain;
+        uniform float gamma;
+
+        vec4 ColorFunc(vec4 C)
+        {
+            C.xyz *= gain;
+            float invgamma = 1.0/gamma;
+            C.xyz = pow (C.xyz, vec3 (invgamma, invgamma, invgamma));
+            return C;
+        }
+    )glsl";
 }
 
 void
@@ -233,15 +233,14 @@ IvGL::create_shaders(void)
     GLint status;
 
     if (!m_vertex_shader) {
-        // clang-format off
-        static const GLchar* vertex_source =
-            "varying vec2 vTexCoord;\n"
-            "void main ()\n"
-            "{\n"
-            "    vTexCoord = gl_MultiTexCoord0.xy;\n"
-            "    gl_Position = ftransform();\n"
-            "}\n";
-        // clang-format on
+        static const GLchar* vertex_source = R"glsl(
+            varying vec2 vTexCoord;
+            void main ()
+            {
+                vTexCoord = gl_MultiTexCoord0.xy;
+                gl_Position = ftransform();
+            }
+        )glsl";
 
         m_vertex_shader = glCreateShader(GL_VERTEX_SHADER);
         glShaderSource(m_vertex_shader, 1, &vertex_source, NULL);
@@ -256,111 +255,118 @@ IvGL::create_shaders(void)
         }
     }
 
-    // clang-format off
-    static const GLchar* fragment_source =
-        "uniform sampler2D imgtex;\n"
-        "varying vec2 vTexCoord;\n"
-        "uniform int startchannel;\n"
-        "uniform int colormode;\n"
+    static const GLchar* fragment_source = R"glsl(
+        uniform sampler2D imgtex;
+        varying vec2 vTexCoord;
+        uniform int startchannel;
+        uniform int colormode;
         // Remember, if imgchannels == 2, second channel would be channel 4 (a).
-        "uniform int imgchannels;\n"
-        "uniform int pixelview;\n"
-        "uniform int linearinterp;\n"
-        "uniform int width;\n"
-        "uniform int height;\n"
-        "vec4 rgba_mode (vec4 C)\n"
-        "{\n"
-        "    if (imgchannels <= 2) {\n"
-        "        if (startchannel == 1)\n"
-        "           return vec4(C.aaa, 1.0);\n"
-        "        return C.rrra;\n"
-        "    }\n"
-        "    return C;\n"
-        "}\n"
-        "vec4 rgb_mode (vec4 C)\n"
-        "{\n"
-        "    if (imgchannels <= 2) {\n"
-        "        if (startchannel == 1)\n"
-        "           return vec4(C.aaa, 1.0);\n"
-        "        return vec4 (C.rrr, 1.0);\n"
-        "    }\n"
-        "    float C2[4];\n"
-        "    C2[0]=C.x; C2[1]=C.y; C2[2]=C.z; C2[3]=C.w;\n"
-        "    return vec4 (C2[startchannel], C2[startchannel+1], C2[startchannel+2], 1.0);\n"
-        "}\n"
-        "vec4 singlechannel_mode (vec4 C)\n"
-        "{\n"
-        "    float C2[4];\n"
-        "    C2[0]=C.x; C2[1]=C.y; C2[2]=C.z; C2[3]=C.w;\n"
-        "    if (startchannel > imgchannels)\n"
-        "        return vec4 (0.0,0.0,0.0,1.0);\n"
-        "    return vec4 (C2[startchannel], C2[startchannel], C2[startchannel], 1.0);\n"
-        "}\n"
-        "vec4 luminance_mode (vec4 C)\n"
-        "{\n"
-        "    if (imgchannels <= 2)\n"
-        "        return vec4 (C.rrr, C.a);\n"
-        "    float lum = dot (C.rgb, vec3(0.2126, 0.7152, 0.0722));\n"
-        "    return vec4 (lum, lum, lum, C.a);\n"
-        "}\n"
-        "float heat_red(float x)\n"
-        "{\n"
-        "    return clamp (mix(0.0, 1.0, (x-0.35)/(0.66-0.35)), 0.0, 1.0) -\n"
-        "           clamp (mix(0.0, 0.5, (x-0.89)/(1.0-0.89)), 0.0, 1.0);\n"
-        "}\n"
-        "float heat_green(float x)\n"
-        "{\n"
-        "    return clamp (mix(0.0, 1.0, (x-0.125)/(0.375-0.125)), 0.0, 1.0) -\n"
-        "           clamp (mix(0.0, 1.0, (x-0.64)/(0.91-0.64)), 0.0, 1.0);\n"
-        "}\n"
-        "vec4 heatmap_mode (vec4 C)\n"
-        "{\n"
-        "    float C2[4];\n"
-        "    C2[0]=C.x; C2[1]=C.y; C2[2]=C.z; C2[3]=C.w;\n"
-        "    return vec4(heat_red(C2[startchannel]),\n"
-        "                heat_green(C2[startchannel]),\n"
-        "                heat_red(1.0-C2[startchannel]),\n"
-        "                1.0);\n"
-        "}\n"
-        "void main ()\n"
-        "{\n"
-        "    vec2 st = vTexCoord;\n"
-        "    float black = 0.0;\n"
-        "    if (pixelview != 0 || linearinterp == 0) {\n"
-        "        vec2 wh = vec2(width,height);\n"
-        "        vec2 onehalf = vec2(0.5,0.5);\n"
-        "        vec2 st_res = st * wh /* + onehalf */ ;\n"
-        "        vec2 st_pix = floor (st_res);\n"
-        "        vec2 st_rem = st_res - st_pix;\n"
-        "        st = (st_pix + onehalf) / wh;\n"
-        "        if (pixelview != 0) {\n"
-        "            if (st.x < 0.0 || st.x >= 1.0 || \n"
-        "                    st.y < 0.0 || st.y >= 1.0 || \n"
-        "                    st_rem.x < 0.05 || st_rem.x >= 0.95 || \n"
-        "                    st_rem.y < 0.05 || st_rem.y >= 0.95)\n"
-        "                black = 1.0;\n"
-        "        }\n"
-        "    }\n"
-        "    vec4 C = texture2D (imgtex, st);\n"
-        "    C = mix (C, vec4(0.05,0.05,0.05,1.0), black);\n"
-        "    if (startchannel < 0)\n"
-        "        C = vec4(0.0,0.0,0.0,1.0);\n"
-        "    else if (colormode == 0)\n" // RGBA
-        "        C = rgba_mode (C);\n"
-        "    else if (colormode == 1)\n" // RGB (i.e., ignore alpha).
-        "        C = rgb_mode (C);\n"
-        "    else if (colormode == 2)\n" // Single channel.
-        "        C = singlechannel_mode (C);\n"
-        "    else if (colormode == 3)\n" // Luminance.
-        "        C = luminance_mode (C);\n"
-        "    else if (colormode == 4)\n" // Heatmap.
-        "        C = heatmap_mode (C);\n"
-        "    if (pixelview != 0)\n"
-        "        C.a = 1.0;\n"
-        "    C = ColorFunc(C);\n"
-        "    gl_FragColor = C;\n"
-        "}\n";
-    // clang-format on
+        uniform int imgchannels;
+        uniform int pixelview;
+        uniform int linearinterp;
+        uniform int width;
+        uniform int height;
+
+        vec4 rgba_mode (vec4 C)
+        {
+            if (imgchannels <= 2) {
+                if (startchannel == 1)
+                return vec4(C.aaa, 1.0);
+                return C.rrra;
+            }
+            return C;
+        }
+
+        vec4 rgb_mode (vec4 C)
+        {
+            if (imgchannels <= 2) {
+                if (startchannel == 1)
+                return vec4(C.aaa, 1.0);
+                return vec4 (C.rrr, 1.0);
+            }
+            float C2[4];
+            C2[0]=C.x; C2[1]=C.y; C2[2]=C.z; C2[3]=C.w;
+            return vec4 (C2[startchannel], C2[startchannel+1], C2[startchannel+2], 1.0);
+        }
+
+        vec4 singlechannel_mode (vec4 C)
+        {
+            float C2[4];
+            C2[0]=C.x; C2[1]=C.y; C2[2]=C.z; C2[3]=C.w;
+            if (startchannel > imgchannels)
+                return vec4 (0.0,0.0,0.0,1.0);
+            return vec4 (C2[startchannel], C2[startchannel], C2[startchannel], 1.0);
+        }
+
+        vec4 luminance_mode (vec4 C)
+        {
+            if (imgchannels <= 2)
+                return vec4 (C.rrr, C.a);
+            float lum = dot (C.rgb, vec3(0.2126, 0.7152, 0.0722));
+            return vec4 (lum, lum, lum, C.a);
+        }
+
+        float heat_red(float x)
+        {
+            return clamp (mix(0.0, 1.0, (x-0.35)/(0.66-0.35)), 0.0, 1.0) -
+                clamp (mix(0.0, 0.5, (x-0.89)/(1.0-0.89)), 0.0, 1.0);
+        }
+
+        float heat_green(float x)
+        {
+            return clamp (mix(0.0, 1.0, (x-0.125)/(0.375-0.125)), 0.0, 1.0) -
+                clamp (mix(0.0, 1.0, (x-0.64)/(0.91-0.64)), 0.0, 1.0);
+        }
+
+        vec4 heatmap_mode (vec4 C)
+        {
+            float C2[4];
+            C2[0]=C.x; C2[1]=C.y; C2[2]=C.z; C2[3]=C.w;
+            return vec4(heat_red(C2[startchannel]),
+                        heat_green(C2[startchannel]),
+                        heat_red(1.0-C2[startchannel]),
+                        1.0);
+        }
+
+        void main ()
+        {
+            vec2 st = vTexCoord;
+            float black = 0.0;
+            if (pixelview != 0 || linearinterp == 0) {
+                vec2 wh = vec2(width,height);
+                vec2 onehalf = vec2(0.5,0.5);
+                vec2 st_res = st * wh /* + onehalf */ ;
+                vec2 st_pix = floor (st_res);
+                vec2 st_rem = st_res - st_pix;
+                st = (st_pix + onehalf) / wh;
+                if (pixelview != 0) {
+                    if (st.x < 0.0 || st.x >= 1.0 || 
+                            st.y < 0.0 || st.y >= 1.0 || 
+                            st_rem.x < 0.05 || st_rem.x >= 0.95 || 
+                            st_rem.y < 0.05 || st_rem.y >= 0.95)
+                        black = 1.0;
+                }
+            }
+            vec4 C = texture2D (imgtex, st);
+            C = mix (C, vec4(0.05,0.05,0.05,1.0), black);
+            if (startchannel < 0)
+                C = vec4(0.0,0.0,0.0,1.0);
+            else if (colormode == 0) // RGBA
+                C = rgba_mode (C);
+            else if (colormode == 1) // RGB (i.e., ignore alpha).
+                C = rgb_mode (C);
+            else if (colormode == 2) // Single channel.
+                C = singlechannel_mode (C);
+            else if (colormode == 3) // Luminance.
+                C = luminance_mode (C);
+            else if (colormode == 4) // Heatmap.
+                C = heatmap_mode (C);
+            if (pixelview != 0)
+                C.a = 1.0;
+            C = ColorFunc(C);
+            gl_FragColor = C;
+        }
+    )glsl";
 
     const char* fragment_sources[] = { "#version 120\n", color_shader,
                                        fragment_source };
@@ -664,12 +670,51 @@ IvGL::paintGL()
         paint_windowguides();
     }
 
+    if (m_selecting) {
+        glPushMatrix();
+        glLoadIdentity();
+
+        glPushAttrib(GL_ENABLE_BIT | GL_CURRENT_BIT);
+        glDisable(GL_TEXTURE_2D);
+        if (m_use_shaders) {
+            glUseProgram(0);
+        }
+
+        glEnable(GL_BLEND);
+        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+        glColor4f(0.2f, 0.5f, 1.0f, 0.3f);  // Light blue fill with transparency
+
+        int w = width();
+        int h = height();
+
+        float x1 = m_select_start.x() - w / 2.0f;
+        float y1 = -(m_select_start.y() - h / 2.0f);
+
+        float x2 = m_select_end.x() - w / 2.0f;
+        float y2 = -(m_select_end.y() - h / 2.0f);
+
+        int left   = std::min(x1, x2);
+        int right  = std::max(x1, x2);
+        int bottom = std::min(y1, y2);
+        int top    = std::max(y1, y2);
+
+        gl_rect(left, bottom, right, top, -0.1f);
+
+        glPopAttrib();
+        glPopMatrix();
+    }
     glPopMatrix();
 
     if (m_viewer.pixelviewOn()) {
         paint_pixelview();
     }
 
+    if (m_viewer.probeviewOn()) {
+        paint_probeview();
+    } else {
+        m_area_probe_text.clear();
+    }
+
     // Show the status info again.
     m_viewer.statusProgress->hide();
     m_viewer.statusViewInfo->show();
@@ -940,6 +985,69 @@ IvGL::paint_pixelview()
 
 
 
+void
+IvGL::paint_probeview()
+{
+    if (!m_current_image)
+        return;
+    IvImage* img = m_current_image;
+    const ImageSpec& spec(img->spec());
+
+    int xw, yw;
+    get_focus_window_pixel(xw, yw);
+
+    glPushMatrix();
+    glLoadIdentity();
+
+    // Set to window pixel units and center the origin
+    glTranslatef(0, 0, -1);  // Push into screen to draw on top
+
+    float closeup_width  = closeupsize * 1.3f;
+    float closeup_height = closeupsize * (0.06f * (spec.nchannels + 1));
+
+    // Position the close-up box
+    const float status_bar_offset = 35.0f;
+    glTranslatef(closeup_width * 0.5f + 5 - width() / 2,
+                 closeup_height * 0.5f + status_bar_offset - height() / 2, 0);
+
+    glPushAttrib(GL_ENABLE_BIT | GL_CURRENT_BIT);
+    glDisable(GL_TEXTURE_2D);
+    if (m_use_shaders)
+        glUseProgram(0);
+    float extraspace = 10 * (1 + spec.nchannels) + 4;
+    glColor4f(0.1f, 0.1f, 0.1f, 0.5f);
+    gl_rect(-0.5f * closeup_width - 2, 0.5f * closeup_height + 10 + 2,
+            0.5f * closeup_width + 2, -0.5f * closeup_height - extraspace,
+            -0.1f);
+    // Draw probe text
+    QFont font;
+
+    int textx    = 9;
+    int texty    = height() - closeup_height - 30;
+    int yspacing = 15;
+
+    if (m_area_probe_text.empty()) {
+        std::ostringstream oss;  // Output stream
+        oss << "Area Probe:\n";
+        for (int i = 0; i < spec.nchannels; ++i)
+            oss << spec.channel_name(i)
+                << ":   [min:  -----, max:  -----, avg:  -----]\n";
+        m_area_probe_text = oss.str();
+    }
+
+    std::istringstream iss(m_area_probe_text);
+    std::string line;
+    while (std::getline(iss, line)) {
+        shadowed_text(textx, texty, 0.0f, line, font);
+        texty += yspacing;
+    }
+
+    glPopAttrib();
+    glPopMatrix();
+}
+
+
+
 void
 IvGL::paint_windowguides()
 {
@@ -1001,6 +1109,8 @@ IvGL::useshader(int tex_width, int tex_height, bool pixelview)
     update_uniforms(tex_width, tex_height, pixelview);
 }
 
+
+
 void
 IvGL::use_program(void)
 {
@@ -1008,6 +1118,8 @@ IvGL::use_program(void)
     print_error("After use program");
 }
 
+
+
 void
 IvGL::update_uniforms(int tex_width, int tex_height, bool pixelview)
 {
@@ -1209,23 +1321,113 @@ IvGL::clamp_view_to_window()
 
 
 
+void
+IvGL::update_area_probe_text()
+{
+    IvImage* img = m_current_image;
+    const ImageSpec& spec(img->spec());
+    // (xw,yw) are the window coordinates of the mouse.
+    int xw, yw;
+    get_focus_window_pixel(xw, yw);
+
+    int x1, y1;
+    get_given_image_pixel(x1, y1, m_select_start.x(), m_select_start.y());
+
+    int x2, y2;
+    get_given_image_pixel(x2, y2, m_select_end.x(), m_select_end.y());
+
+    float scale_x  = 1.0f;
+    float scale_y  = 1.0f;
+    float rotate_z = 0.0f;
+    float x1_img   = x1;
+    float y1_img   = y1;
+    float x2_img   = x2;
+    float y2_img   = y2;
+
+    handle_orientation(img->orientation(), spec.width, spec.height, scale_x,
+                       scale_y, rotate_z, x1_img, y1_img, true);
+    handle_orientation(img->orientation(), spec.width, spec.height, scale_x,
+                       scale_y, rotate_z, x2_img, y2_img, true);
+
+    x1_img = clamp<int>(x1_img, 0, spec.width - 1);
+    x2_img = clamp<int>(x2_img, 0, spec.width - 1);
+    y1_img = clamp<int>(y1_img, 0, spec.height - 1);
+    y2_img = clamp<int>(y2_img, 0, spec.height - 1);
+
+    int xmin = std::min(x1_img, x2_img);
+    int xmax = std::max(x1_img, x2_img);
+    int ymin = std::min(y1_img, y2_img);
+    int ymax = std::max(y1_img, y2_img);
+
+    // Min and max
+    std::vector<float> min_vals(spec.nchannels,
+                                std::numeric_limits<float>::max());
+    std::vector<float> max_vals(spec.nchannels,
+                                std::numeric_limits<float>::lowest());
+    std::vector<double> sums(spec.nchannels, 0.0);
+    int count = 0;
+
+    // loop through each pixel
+    float* fpixel = OIIO_ALLOCA(float, spec.nchannels);
+    for (int y = ymin; y <= ymax; ++y) {
+        for (int x = xmin; x <= xmax; ++x) {
+            img->getpixel(x + spec.x, y + spec.y, fpixel);
+            for (int c = 0; c < spec.nchannels; ++c) {
+                min_vals[c] = std::min(min_vals[c], fpixel[c]);
+                max_vals[c] = std::max(max_vals[c], fpixel[c]);
+                sums[c] += fpixel[c];
+            }
+            ++count;
+        }
+    }
+
+    QString result = "Area Probe:\n";
+    for (int c = 0; c < spec.nchannels; ++c) {
+        float avg = (count > 0) ? static_cast<float>(sums[c] / count) : 0.0f;
+        result += QString("%1: [min: %2  max: %3  avg: %4]\n")
+                      .arg(QString::fromStdString(spec.channel_name(c))
+                               .leftJustified(5))
+                      .arg(min_vals[c], 6, 'f', 3)
+                      .arg(max_vals[c], 6, 'f', 3)
+                      .arg(avg, 6, 'f', 3);
+    }
+
+    m_area_probe_text = result.toStdString();
+}
+
+
+
 void
 IvGL::mousePressEvent(QMouseEvent* event)
 {
     remember_mouse(event->pos());
     int mousemode = m_viewer.mouseModeComboBox->currentIndex();
+    bool areaMode = m_viewer.areaSampleMode();
     bool Alt      = (event->modifiers() & Qt::AltModifier);
     m_drag_button = event->button();
     if (!m_mouse_activation) {
         switch (event->button()) {
         case Qt::LeftButton:
-            if (mousemode == ImageViewer::MouseModeZoom && !Alt)
+            if (areaMode) {
+                m_select_start = event->pos();
+                m_select_end   = m_select_start;
+                m_selecting    = true;
+                parent_t::update();
+            } else if (mousemode == ImageViewer::MouseModeSelect && !Alt
+                       && areaMode) {
+                std::cerr << areaMode;
+                m_select_start = event->pos();
+                m_select_end   = m_select_start;
+                m_selecting    = true;
+                parent_t::update();
+            } else if (mousemode == ImageViewer::MouseModeZoom && !Alt
+                       && !areaMode) {
                 m_viewer.zoomIn(true);  // Animated zoom for mouse clicks
-            else
+            } else
                 m_dragging = true;
             return;
         case Qt::RightButton:
-            if (mousemode == ImageViewer::MouseModeZoom && !Alt)
+            if (mousemode == ImageViewer::MouseModeZoom && !Alt && !areaMode)
                 m_viewer.zoomOut(true);  // Animated zoom for mouse clicks
             else
                 m_dragging = true;
@@ -1253,6 +1455,14 @@ IvGL::mouseReleaseEvent(QMouseEvent* event)
     remember_mouse(event->pos());
     m_drag_button = Qt::NoButton;
     m_dragging    = false;
+    if (m_selecting) {
+        m_select_end = event->pos();
+        m_selecting  = false;
+        update_area_probe_text();
+        m_select_start = QPoint();
+        m_select_end   = QPoint();
+        parent_t::update();
+    }
     parent_t::mouseReleaseEvent(event);
 }
 
@@ -1262,6 +1472,20 @@ void
 IvGL::mouseMoveEvent(QMouseEvent* event)
 {
     QPoint pos = event->pos();
+
+    // Area probe override
+    if (m_viewer.areaSampleMode() && m_selecting) {
+        m_select_end = event->pos();
+        update_area_probe_text();
+        remember_mouse(pos);
+        parent_t::update();
+        if (m_viewer.pixelviewOn()) {
+            parent_t::update();
+        }
+        parent_t::mouseMoveEvent(event);
+        return;
+    }
+
     // FIXME - there's probably a better Qt way than tracking the button
     // myself.
     bool Alt      = (event->modifiers() & Qt::AltModifier);
@@ -1312,6 +1536,10 @@ IvGL::mouseMoveEvent(QMouseEvent* event)
     } else if (do_wipe) {
         // FIXME -- unimplemented
     } else if (do_select) {
+        if (m_selecting) {
+            m_select_end = event->pos();
+            parent_t::update();
+        }
         // FIXME -- unimplemented
     } else if (do_annotate) {
         // FIXME -- unimplemented
@@ -1323,6 +1551,7 @@ IvGL::mouseMoveEvent(QMouseEvent* event)
 }
 
 
+
 void
 IvGL::wheelEvent(QWheelEvent* event)
 {
@@ -1359,6 +1588,33 @@ IvGL::get_focus_window_pixel(int& x, int& y)
 
 
 
+void
+IvGL::get_given_image_pixel(int& x, int& y, int mouseX, int mouseY)
+{
+    int w = width(), h = height();
+    float z = m_zoom;
+    // left,top,right,bottom are the borders of the visible window, in
+    // pixel coordinates
+    float left   = m_centerx - 0.5 * w / z;
+    float top    = m_centery - 0.5 * h / z;
+    float right  = m_centerx + 0.5 * w / z;
+    float bottom = m_centery + 0.5 * h / z;
+    // normx,normy are the position of the mouse, in normalized (i.e. [0..1])
+    // visible window coordinates.
+    float normx = (float)(mouseX + 0.5f) / w;
+    float normy = (float)(mouseY + 0.5f) / h;
+    // imgx,imgy are the position of the mouse, in pixel coordinates
+    float imgx = OIIO::lerp(left, right, normx);
+    float imgy = OIIO::lerp(top, bottom, normy);
+    // So finally x,y are the coordinates of the image pixel (on [0,res-1])
+    // underneath the mouse cursor.
+    //FIXME: Shouldn't this take image rotation into account?
+    x = (int)floorf(imgx);
+    y = (int)floorf(imgy);
+}
+
+
+
 void
 IvGL::get_focus_image_pixel(int& x, int& y)
 {
@@ -1397,6 +1653,7 @@ IvGL::get_focus_image_pixel(int& x, int& y)
 }
 
 
+
 void
 IvGL::print_shader_log(std::ostream& out, const GLuint shader_id)
 {
@@ -1643,12 +1900,16 @@ IvGL::is_too_big(float width, float height)
     return tiles > m_texbufs.size();
 }
 
+
+
 void
 IvGL::update_state(void)
 {
     create_shaders();
 }
 
+
+
 void
 IvGL::print_error(const char* msg)
 {
