@@ -16,107 +16,103 @@
 OIIO_NAMESPACE_BEGIN
 
 class ImageCache;
+class TextureSystemImpl;
 class Filter1D;
 
-namespace pvt {
-
-class TextureSystemImpl;
-
 #ifndef OPENIMAGEIO_IMAGECACHE_PVT_H
-class ImageCacheImpl;
 class ImageCacheFile;
 class ImageCacheTile;
 class ImageCacheTileRef;
+class ImageCacheTileID;
+class ImageCacheImpl;
 #endif
 
 
 
 /// Working implementation of the abstract TextureSystem class.
-///
-class TextureSystemImpl final : public TextureSystem {
+class TextureSystemImpl {
 public:
-    typedef ImageCacheFile TextureFile;
+    using TextureHandle = TextureSystem::TextureHandle;
+    using Perthread     = TextureSystem::Perthread;
+    using TextureFile   = ImageCacheFile;
 
-    TextureSystemImpl(ImageCache* imagecache);
-    ~TextureSystemImpl() override;
+    TextureSystemImpl(std::shared_ptr<ImageCache> imagecache = {});
+    ~TextureSystemImpl();
 
-    bool attribute(string_view name, TypeDesc type, const void* val) override;
-    bool attribute(string_view name, int val) override
+    bool attribute(string_view name, TypeDesc type, const void* val);
+    bool attribute(string_view name, int val)
     {
-        return attribute(name, TypeDesc::INT, &val);
+        return attribute(name, TypeInt, &val);
     }
-    bool attribute(string_view name, float val) override
+    bool attribute(string_view name, float val)
     {
-        return attribute(name, TypeDesc::FLOAT, &val);
+        return attribute(name, TypeFloat, &val);
     }
-    bool attribute(string_view name, double val) override
+    bool attribute(string_view name, double val)
     {
         float f = (float)val;
-        return attribute(name, TypeDesc::FLOAT, &f);
+        return attribute(name, TypeFloat, &f);
     }
-    bool attribute(string_view name, string_view val) override
+    bool attribute(string_view name, string_view val)
     {
         std::string valstr(val);
         const char* s = valstr.c_str();
-        return attribute(name, TypeDesc::STRING, &s);
+        return attribute(name, TypeString, &s);
     }
 
-    TypeDesc getattributetype(string_view name) const override;
+    TypeDesc getattributetype(string_view name) const;
 
-    bool getattribute(string_view name, TypeDesc type,
-                      void* val) const override;
-    bool getattribute(string_view name, int& val) const override
+    bool getattribute(string_view name, TypeDesc type, void* val) const;
+    bool getattribute(string_view name, int& val) const
     {
-        return getattribute(name, TypeDesc::INT, &val);
+        return getattribute(name, TypeInt, &val);
     }
-    bool getattribute(string_view name, float& val) const override
+    bool getattribute(string_view name, float& val) const
     {
-        return getattribute(name, TypeDesc::FLOAT, &val);
+        return getattribute(name, TypeFloat, &val);
     }
-    bool getattribute(string_view name, double& val) const override
+    bool getattribute(string_view name, double& val) const
     {
         float f;
-        bool ok = getattribute(name, TypeDesc::FLOAT, &f);
+        bool ok = getattribute(name, TypeFloat, &f);
         if (ok)
             val = f;
         return ok;
     }
-    bool getattribute(string_view name, char** val) const override
+    bool getattribute(string_view name, char** val) const
     {
-        return getattribute(name, TypeDesc::STRING, val);
+        return getattribute(name, TypeString, val);
     }
-    bool getattribute(string_view name, std::string& val) const override
+    bool getattribute(string_view name, std::string& val) const
     {
         const char* s;
-        bool ok = getattribute(name, TypeDesc::STRING, &s);
+        bool ok = getattribute(name, TypeString, &s);
         if (ok)
             val = s;
         return ok;
     }
 
-
     // Retrieve options
     void get_commontoworld(Imath::M44f& result) const { result = m_Mc2w; }
 
-    Perthread* get_perthread_info(Perthread* thread_info = NULL) override
+    Perthread* get_perthread_info(Perthread* thread_info = NULL)
     {
         return (Perthread*)m_imagecache->get_perthread_info(
             (ImageCachePerThreadInfo*)thread_info);
     }
-    Perthread* create_thread_info() override
+    Perthread* create_thread_info()
     {
         OIIO_ASSERT(m_imagecache);
         return (Perthread*)m_imagecache->create_thread_info();
     }
-    void destroy_thread_info(Perthread* threadinfo) override
+    void destroy_thread_info(Perthread* threadinfo)
     {
         OIIO_ASSERT(m_imagecache);
         m_imagecache->destroy_thread_info((ImageCachePerThreadInfo*)threadinfo);
     }
 
-    TextureHandle*
-    get_texture_handle(ustring filename, Perthread* thread,
-                       const TextureOpt* options = nullptr) override
+    TextureHandle* get_texture_handle(ustring filename, Perthread* thread,
+                                      const TextureOpt* options = nullptr)
     {
         PerThreadInfo* thread_info = thread
                                          ? ((PerThreadInfo*)thread)
@@ -124,225 +120,134 @@ public:
         return (TextureHandle*)find_texturefile(filename, thread_info, options);
     }
 
-    bool good(TextureHandle* texture_handle) override
+    bool good(TextureHandle* texture_handle)
     {
         return texture_handle && !((TextureFile*)texture_handle)->broken();
     }
 
-    ustring filename_from_handle(TextureHandle* handle) override
+    ustring filename_from_handle(TextureHandle* handle)
     {
         return handle ? ((ImageCache::ImageHandle*)handle)->filename()
                       : ustring();
     }
 
-    int get_colortransform_id(ustring fromspace,
-                              ustring tospace) const override;
-    int get_colortransform_id(ustringhash fromspace,
-                              ustringhash tospace) const override;
+    int get_colortransform_id(ustring fromspace, ustring tospace) const;
+    int get_colortransform_id(ustringhash fromspace, ustringhash tospace) const;
 
     bool texture(ustring filename, TextureOpt& options, float s, float t,
                  float dsdx, float dtdx, float dsdy, float dtdy, int nchannels,
                  float* result, float* dresultds = NULL,
-                 float* dresultdt = NULL) override;
+                 float* dresultdt = NULL);
     bool texture(TextureHandle* texture_handle, Perthread* thread_info,
                  TextureOpt& options, float s, float t, float dsdx, float dtdx,
                  float dsdy, float dtdy, int nchannels, float* result,
-                 float* dresultds = NULL, float* dresultdt = NULL) override;
+                 float* dresultds = NULL, float* dresultdt = NULL);
     bool texture(ustring filename, TextureOptBatch& options, Tex::RunMask mask,
                  const float* s, const float* t, const float* dsdx,
                  const float* dtdx, const float* dsdy, const float* dtdy,
                  int nchannels, float* result, float* dresultds = nullptr,
-                 float* dresultdt = nullptr) override;
+                 float* dresultdt = nullptr);
     bool texture(TextureHandle* texture_handle, Perthread* thread_info,
                  TextureOptBatch& options, Tex::RunMask mask, const float* s,
                  const float* t, const float* dsdx, const float* dtdx,
                  const float* dsdy, const float* dtdy, int nchannels,
                  float* result, float* dresultds = nullptr,
-                 float* dresultdt = nullptr) override;
-    bool texture(ustring filename, TextureOptions& options, Runflag* runflags,
-                 int beginactive, int endactive, VaryingRef<float> s,
-                 VaryingRef<float> t, VaryingRef<float> dsdx,
-                 VaryingRef<float> dtdx, VaryingRef<float> dsdy,
-                 VaryingRef<float> dtdy, int nchannels, float* result,
-                 float* dresultds = NULL, float* dresultdt = NULL) override;
-    bool texture(TextureHandle* texture_handle, Perthread* thread_info,
-                 TextureOptions& options, Runflag* runflags, int beginactive,
-                 int endactive, VaryingRef<float> s, VaryingRef<float> t,
-                 VaryingRef<float> dsdx, VaryingRef<float> dtdx,
-                 VaryingRef<float> dsdy, VaryingRef<float> dtdy, int nchannels,
-                 float* result, float* dresultds = NULL,
-                 float* dresultdt = NULL) override;
+                 float* dresultdt = nullptr);
 
     bool texture3d(ustring filename, TextureOpt& options, V3fParam P,
                    V3fParam dPdx, V3fParam dPdy, V3fParam dPdz, int nchannels,
                    float* result, float* dresultds = NULL,
-                   float* dresultdt = NULL, float* dresultdr = NULL) override;
+                   float* dresultdt = NULL, float* dresultdr = NULL);
     bool texture3d(TextureHandle* texture_handle, Perthread* thread_info,
                    TextureOpt& options, V3fParam P, V3fParam dPdx,
                    V3fParam dPdy, V3fParam dPdz, int nchannels, float* result,
                    float* dresultds = NULL, float* dresultdt = NULL,
-                   float* dresultdr = NULL) override;
+                   float* dresultdr = NULL);
     bool texture3d(ustring filename, TextureOptBatch& options,
                    Tex::RunMask mask, const float* P, const float* dPdx,
                    const float* dPdy, const float* dPdz, int nchannels,
                    float* result, float* dresultds = nullptr,
-                   float* dresultdt = nullptr,
-                   float* dresultdr = nullptr) override;
+                   float* dresultdt = nullptr, float* dresultdr = nullptr);
     bool texture3d(TextureHandle* texture_handle, Perthread* thread_info,
                    TextureOptBatch& options, Tex::RunMask mask, const float* P,
                    const float* dPdx, const float* dPdy, const float* dPdz,
                    int nchannels, float* result, float* dresultds = nullptr,
-                   float* dresultdt = nullptr,
-                   float* dresultdr = nullptr) override;
-    bool texture3d(ustring filename, TextureOptions& options, Runflag* runflags,
-                   int beginactive, int endactive, VaryingRef<Imath::V3f> P,
-                   VaryingRef<Imath::V3f> dPdx, VaryingRef<Imath::V3f> dPdy,
-                   VaryingRef<Imath::V3f> dPdz, int nchannels, float* result,
-                   float* dresultds = NULL, float* dresultdt = NULL,
-                   float* dresultdr = NULL) override;
-    bool texture3d(TextureHandle* texture_handle, Perthread* thread_info,
-                   TextureOptions& options, Runflag* runflags, int beginactive,
-                   int endactive, VaryingRef<Imath::V3f> P,
-                   VaryingRef<Imath::V3f> dPdx, VaryingRef<Imath::V3f> dPdy,
-                   VaryingRef<Imath::V3f> dPdz, int nchannels, float* result,
-                   float* dresultds = NULL, float* dresultdt = NULL,
-                   float* dresultdr = NULL) override;
-
-    bool shadow(ustring /*filename*/, TextureOpt& /*options*/, V3fParam /*P*/,
-                V3fParam /*dPdx*/, V3fParam /*dPdy*/, float* /*result*/,
-                float* /*dresultds*/, float* /*dresultdt*/) override
-    {
-        return false;
-    }
-    bool shadow(TextureHandle* /*texture_handle*/, Perthread* /*thread_info*/,
-                TextureOpt& /*options*/, V3fParam /*P*/, V3fParam /*dPdx*/,
-                V3fParam /*dPdy*/, float* /*result*/, float* /*dresultds*/,
-                float* /*dresultdt*/) override
-    {
-        return false;
-    }
-    bool shadow(ustring /*filename*/, TextureOptBatch& /*options*/,
-                Tex::RunMask /*mask*/, const float* /*P*/,
-                const float* /*dPdx*/, const float* /*dPdy*/, float* /*result*/,
-                float* /*dresultds*/, float* /*dresultdt*/) override
-    {
-        return false;
-    }
-    bool shadow(TextureHandle* /*texture_handle*/, Perthread* /*thread_info*/,
-                TextureOptBatch& /*options*/, Tex::RunMask /*mask*/,
-                const float* /*P*/, const float* /*dPdx*/,
-                const float* /*dPdy*/, float* /*result*/, float* /*dresultds*/,
-                float* /*dresultdt*/) override
-    {
-        return false;
-    }
-    bool shadow(ustring /*filename*/, TextureOptions& /*options*/,
-                Runflag* /*runflags*/, int /*beginactive*/, int /*endactive*/,
-                VaryingRef<Imath::V3f> /*P*/, VaryingRef<Imath::V3f> /*dPdx*/,
-                VaryingRef<Imath::V3f> /*dPdy*/, float* /*result*/,
-                float* /*dresultds*/, float* /*dresultdt*/) override
-    {
-        return false;
-    }
-    bool shadow(TextureHandle* /*texture_handle*/, Perthread* /*thread_info*/,
-                TextureOptions& /*options*/, Runflag* /*runflags*/,
-                int /*beginactive*/, int /*endactive*/,
-                VaryingRef<Imath::V3f> /*P*/, VaryingRef<Imath::V3f> /*dPdx*/,
-                VaryingRef<Imath::V3f> /*dPdy*/, float* /*result*/,
-                float* /*dresultds*/, float* /*dresultdt*/) override
-    {
-        return false;
-    }
-
+                   float* dresultdt = nullptr, float* dresultdr = nullptr);
 
     bool environment(ustring filename, TextureOpt& options, V3fParam R,
                      V3fParam dRdx, V3fParam dRdy, int nchannels, float* result,
-                     float* dresultds = NULL, float* dresultdt = NULL) override;
+                     float* dresultds = NULL, float* dresultdt = NULL);
     bool environment(TextureHandle* texture_handle, Perthread* thread_info,
                      TextureOpt& options, V3fParam R, V3fParam dRdx,
                      V3fParam dRdy, int nchannels, float* result,
-                     float* dresultds = NULL, float* dresultdt = NULL) override;
+                     float* dresultds = NULL, float* dresultdt = NULL);
     bool environment(ustring filename, TextureOptBatch& options,
                      Tex::RunMask mask, const float* R, const float* dRdx,
                      const float* dRdy, int nchannels, float* result,
-                     float* dresultds = nullptr,
-                     float* dresultdt = nullptr) override;
+                     float* dresultds = nullptr, float* dresultdt = nullptr);
     bool environment(TextureHandle* texture_handle, Perthread* thread_info,
                      TextureOptBatch& options, Tex::RunMask mask,
                      const float* R, const float* dRdx, const float* dRdy,
                      int nchannels, float* result, float* dresultds = nullptr,
-                     float* dresultdt = nullptr) override;
-    bool environment(ustring filename, TextureOptions& options,
-                     Runflag* runflags, int beginactive, int endactive,
-                     VaryingRef<Imath::V3f> R, VaryingRef<Imath::V3f> dRdx,
-                     VaryingRef<Imath::V3f> dRdy, int nchannels, float* result,
-                     float* dresultds = NULL, float* dresultdt = NULL) override;
-    bool environment(TextureHandle* texture_handle, Perthread* thread_info,
-                     TextureOptions& options, Runflag* runflags,
-                     int beginactive, int endactive, VaryingRef<Imath::V3f> R,
-                     VaryingRef<Imath::V3f> dRdx, VaryingRef<Imath::V3f> dRdy,
-                     int nchannels, float* result, float* dresultds = NULL,
-                     float* dresultdt = NULL) override;
+                     float* dresultdt = nullptr);
 
-    std::string resolve_filename(const std::string& filename) const override;
+    std::string resolve_filename(const std::string& filename) const;
 
     bool get_texture_info(ustring filename, int subimage, ustring dataname,
-                          TypeDesc datatype, void* data) override;
+                          TypeDesc datatype, void* data);
     bool get_texture_info(TextureHandle* texture_handle, Perthread* thread_info,
                           int subimage, ustring dataname, TypeDesc datatype,
-                          void* data) override;
+                          void* data);
 
-    bool get_imagespec(ustring filename, int subimage,
-                       ImageSpec& spec) override;
+    bool get_imagespec(ustring filename, ImageSpec& spec, int subimage);
     bool get_imagespec(TextureHandle* texture_handle, Perthread* thread_info,
-                       int subimage, ImageSpec& spec) override;
+                       ImageSpec& spec, int subimage);
 
-    const ImageSpec* imagespec(ustring filename, int subimage = 0) override;
+    const ImageSpec* imagespec(ustring filename, int subimage = 0);
     const ImageSpec* imagespec(TextureHandle* texture_handle,
-                               Perthread* thread_info = NULL,
-                               int subimage           = 0) override;
+                               Perthread* thread_info = NULL, int subimage = 0);
 
     bool get_texels(ustring filename, TextureOpt& options, int miplevel,
                     int xbegin, int xend, int ybegin, int yend, int zbegin,
                     int zend, int chbegin, int chend, TypeDesc format,
-                    void* result) override;
+                    void* result);
     bool get_texels(TextureHandle* texture_handle, Perthread* thread_info,
                     TextureOpt& options, int miplevel, int xbegin, int xend,
                     int ybegin, int yend, int zbegin, int zend, int chbegin,
-                    int chend, TypeDesc format, void* result) override;
+                    int chend, TypeDesc format, void* result);
 
-    bool is_udim(ustring filename) override;
-    bool is_udim(TextureHandle* udimfile) override;
-    TextureHandle* resolve_udim(ustring filename, float s, float t) override;
+    bool is_udim(ustring filename);
+    bool is_udim(TextureHandle* udimfile);
+    TextureHandle* resolve_udim(ustring filename, float s, float t);
     TextureHandle* resolve_udim(TextureHandle* udimfile, Perthread* thread_info,
-                                float s, float t) override;
+                                float s, float t);
     void inventory_udim(ustring udimpattern, std::vector<ustring>& filenames,
-                        int& nutiles, int& nvtiles) override;
+                        int& nutiles, int& nvtiles);
     void inventory_udim(TextureHandle* udimfile, Perthread* thread_info,
                         std::vector<ustring>& filenames, int& nutiles,
-                        int& nvtiles) override;
+                        int& nvtiles);
 
-    bool has_error() const override;
-    std::string geterror(bool clear = true) const override;
-    std::string getstats(int level = 1, bool icstats = true) const override;
-    void reset_stats() override;
+    bool has_error() const;
+    std::string geterror(bool clear = true) const;
+    std::string getstats(int level = 1, bool icstats = true) const;
+    void reset_stats();
 
-    void invalidate(ustring filename, bool force) override;
-    void invalidate_all(bool force = false) override;
-    void close(ustring filename) override;
-    void close_all() override;
+    void invalidate(ustring filename, bool force);
+    void invalidate_all(bool force = false);
+    void close(ustring filename);
+    void close_all();
 
-    void operator delete(void* todel) { ::delete ((char*)todel); }
+    // void operator delete(void* todel) { ::delete ((char*)todel); }
 
     typedef bool (*wrap_impl)(int& coord, int origin, int width);
 
     /// Return an opaque, non-owning pointer to the underlying ImageCache
     /// (if there is one).
-    ImageCache* imagecache() const override { return m_imagecache; }
+    std::shared_ptr<ImageCache> imagecache() const { return m_imagecache_sp; }
 
-private:
-    typedef ImageCacheTileRef TileRef;
-    typedef ImageCachePerThreadInfo PerThreadInfo;
+    // private:
+    using TileRef       = ImageCacheTileRef;
+    using PerThreadInfo = ImageCachePerThreadInfo;
 
     void init();
 
@@ -550,10 +455,12 @@ private:
     void visualize_ellipse(const std::string& name, float dsdx, float dtdx,
                            float dsdy, float dtdy, float sblur, float tblur);
 
+    std::shared_ptr<ImageCache> m_imagecache_sp;
     ImageCacheImpl* m_imagecache = nullptr;
-    bool m_imagecache_owner      = false;  ///< True if we own the ImageCache
-    Imath::M44f m_Mw2c;                    ///< world-to-"common" matrix
-    Imath::M44f m_Mc2w;                    ///< common-to-world matrix
+    uint64_t m_id;                    // A unique ID for this TextureSystem
+    Imath::M44f m_Mw2c;               ///< world-to-"common" matrix
+    Imath::M44f m_Mc2w;               ///< common-to-world matrix
+    bool m_imagecache_owner = false;  ///< True if we own the ImageCache
     bool m_gray_to_rgb;       ///< automatically copy gray to rgb channels?
     bool m_flip_t;            ///< Flip direction of t coord?
     int m_max_tile_channels;  ///< narrow tile ID channel range when
@@ -568,9 +475,6 @@ private:
         StochasticStrategy_Texel = 4   // single FIS texel probe
     };
 
-    /// Saved error string, per-thread
-    ///
-    mutable thread_specific_ptr<std::string> m_errormessage;
     std::unique_ptr<Filter1D> hq_filter;  // Better filter for magnification
     int m_statslevel;
     friend class TextureSystem;
@@ -647,9 +551,6 @@ TextureSystemImpl::st_to_texel(float s, float t, TextureFile& texturefile,
 }
 
 
-
-}  // end namespace pvt
-
 OIIO_NAMESPACE_END
 
 #endif  // OPENIMAGEIO_TEXTURE_PVT_H
